'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.invertValidity = exports.invertValidators = exports.getFieldFromState = exports.getForm = exports.isInvalid = exports.isValid = exports.getValidity = exports.getValue = exports.getEventValue = exports.isTouched = exports.isPristine = exports.isMulti = exports.isFocused = undefined;

var _endsWith = require('lodash/endsWith');

var _endsWith2 = _interopRequireDefault(_endsWith);

var _mapValues = require('../utils/map-values');

var _mapValues2 = _interopRequireDefault(_mapValues);

var _isPlainObject = require('lodash/isPlainObject');

var _isPlainObject2 = _interopRequireDefault(_isPlainObject);

var _every = require('lodash/every');

var _every2 = _interopRequireDefault(_every);

var _some = require('lodash/some');

var _some2 = _interopRequireDefault(_some);

var _findKey = require('lodash/findKey');

var _findKey2 = _interopRequireDefault(_findKey);

var _get = require('lodash/get');

var _get2 = _interopRequireDefault(_get);

var _toPath = require('lodash/toPath');

var _toPath2 = _interopRequireDefault(_toPath);

var _startsWith = require('lodash/startsWith');

var _startsWith2 = _interopRequireDefault(_startsWith);

var _memoize = require('lodash/memoize');

var _memoize2 = _interopRequireDefault(_memoize);

var _formReducer = require('../reducers/form-reducer');

var _flatten = require('./flatten');

var _flatten2 = _interopRequireDefault(_flatten);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function isMulti(model) {
  return (0, _endsWith2.default)(model, '[]');
}

function isFocused(field) {
  return field && field.focus;
}

function isPristine(field) {
  return field ? field.pristine : _formReducer.initialFieldState.pristine;
}

function isTouched(field) {
  return field ? field.touched : _formReducer.initialFieldState.touched;
}

function isEvent(event) {
  return !!(event && event.stopPropagation && event.preventDefault);
}

function getEventValue(event) {
  if (!event.target) {
    if (!event.nativeEvent) {
      return undefined;
    }

    return event.nativeEvent.text;
  }

  if (event.target.multiple) {
    return [].concat(_toConsumableArray(event.target.selectedOptions)).map(function (option) {
      return option.value;
    });
  }

  return event.target.value;
}

function getValue(value) {
  return isEvent(value) ? getEventValue(value) : value;
}

var getFormStateKey = (0, _memoize2.default)(function (state, model) {
  var flatState = (0, _flatten2.default)(state);

  var formStateKey = (0, _findKey2.default)(flatState, function (value) {
    return value && value.model && (0, _startsWith2.default)(model, value.model);
  });

  return formStateKey;
});

function getForm(state, model) {
  var formStateKey = getFormStateKey(state, model);

  return (0, _get2.default)(state, formStateKey);
}

function getFieldFromState(state, model) {
  var form = getForm(state, model);

  if (!form) return null;

  return (0, _formReducer.getField)(form, (0, _toPath2.default)(model).slice(1));
}

function getValidity(validators, value) {
  var modelValue = getValue(value);

  if (typeof validators === 'function') {
    return validators(modelValue);
  }

  return (0, _mapValues2.default)(validators, function (validator) {
    return getValidity(validator, modelValue);
  });
}

function invertValidators(validators) {
  if (typeof validators === 'function') {
    return function (val) {
      return !validators(val);
    };
  }

  return (0, _mapValues2.default)(validators, invertValidators);
}

function invertValidity(validity) {
  if ((0, _isPlainObject2.default)(validity)) {
    return (0, _mapValues2.default)(validity, invertValidity);
  }

  return !validity;
}

function isValid(validity) {
  if ((0, _isPlainObject2.default)(validity)) {
    return (0, _every2.default)(validity, isValid);
  }

  return !!validity;
}

function isInvalid(errors) {
  if ((0, _isPlainObject2.default)(errors)) {
    return (0, _some2.default)(errors, isInvalid);
  }

  return !!errors;
}

exports.isFocused = isFocused;
exports.isMulti = isMulti;
exports.isPristine = isPristine;
exports.isTouched = isTouched;
exports.getEventValue = getEventValue;
exports.getValue = getValue;
exports.getValidity = getValidity;
exports.isValid = isValid;
exports.isInvalid = isInvalid;
exports.getForm = getForm;
exports.getFieldFromState = getFieldFromState;
exports.invertValidators = invertValidators;
exports.invertValidity = invertValidity;